@@ -117,3 +117,12 @@ CHANNEL_STATES = {
 
 EFFECT_RANDOM = "random"
 EFFECT_MUSIC = "music"
+
+# Addressable limits
+SEGMENTS_MAX: Final = 2048
+PIXELS_MAX: Final = 2048
+PIXELS_PER_SEGMENT_MAX: Final = 300
+
+MUSIC_SEGMENTS_MAX: Final = 64
+MUSIC_PIXELS_MAX: Final = 960
+MUSIC_PIXELS_PER_SEGMENT_MAX: Final = 150
