@@ -24,6 +24,7 @@ from .const import (
     LevelWriteMode,
     MultiColorEffects,
 )
+from .timer import LedTimer
 from .utils import utils, white_levels_to_scaled_color_temp
 
 
@@ -75,6 +76,7 @@ PROTOCOL_LEDENET_ORIGINAL_CCT = "LEDENET_ORIGINAL_CCT"
 PROTOCOL_LEDENET_9BYTE = "LEDENET"
 PROTOCOL_LEDENET_9BYTE_AUTO_ON = "LEDENET_AUTO_ON"
 PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS = "LEDENET_DIMMABLE_EFFECTS"
+PROTOCOL_LEDENET_SOCKET = "LEDENET_SOCKET"
 PROTOCOL_LEDENET_8BYTE = "LEDENET_8BYTE"  # Previously was called None
 PROTOCOL_LEDENET_8BYTE_AUTO_ON = "LEDENET_8BYTES_AUTO_ON"
 PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS = "LEDENET_8BYTE_DIMMABLE_EFFECTS"
@@ -100,19 +102,19 @@ LEDENET_ADDRESSABLE_STATE_RESPONSE_LEN = 25
 LEDENET_A1_DEVICE_CONFIG_RESPONSE_LEN = 12
 LEDENET_DEVICE_CONFIG_RESPONSE_LEN = 11
 LEDENET_REMOTE_CONFIG_RESPONSE_LEN = 14  # 2b 03 00 00 00 00 29 00 00 00 00 00 00 57
-LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN = 12  # 10 14 16 01 02 10 26 20 07 00 0f a9
+LEDENET_TIME_RESPONSE_LEN = 12  # 10 14 16 01 02 10 26 20 07 00 0f a9
+LEDENET_TIMERS_8BYTE_RESPONSE_LEN = 88
+LEDENET_TIMERS_9BYTE_RESPONSE_LEN = 94
 
 MSG_ORIGINAL_POWER_STATE = "original_power_state"
 MSG_ORIGINAL_STATE = "original_state"
-
 MSG_POWER_RESTORE_STATE = "power_restore_state"
 MSG_POWER_STATE = "power_state"
 MSG_STATE = "state"
-
 MSG_TIME = "time"
+MSG_TIMERS = "timers"
 MSG_MUSIC_MODE_STATE = "music_mode_state"
 MSG_ADDRESSABLE_STATE = "addressable_state"
-
 MSG_DEVICE_CONFIG = "device_config"
 MSG_A1_DEVICE_CONFIG = "a1_device_config"
 MSG_REMOTE_CONFIG = "remote_config"
@@ -123,6 +125,9 @@ MSG_UNIQUE_START = {
     (0xF0, 0x11): MSG_TIME,
     (0x0F, 0x11): MSG_TIME,
     (0x00, 0x11): MSG_TIME,
+    (0xF0, 0x22): MSG_TIMERS,
+    (0x0F, 0x22): MSG_TIMERS,
+    (0x00, 0x22): MSG_TIMERS,
     (0xF0, 0x71): MSG_POWER_STATE,
     (0x0F, 0x71): MSG_POWER_STATE,
     (0x00, 0x71): MSG_POWER_STATE,
@@ -141,7 +146,7 @@ MSG_UNIQUE_START = {
 }
 
 MSG_LENGTHS = {
-    MSG_TIME: LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN,
+    MSG_TIME: LEDENET_TIME_RESPONSE_LEN,
     MSG_REMOTE_CONFIG: LEDENET_REMOTE_CONFIG_RESPONSE_LEN,
     MSG_MUSIC_MODE_STATE: LEDNET_MUSIC_MODE_RESPONSE_LEN,
     MSG_POWER_STATE: LEDENET_POWER_RESPONSE_LEN,
@@ -463,6 +468,8 @@ class ProtocolBase:
         msg_type = _message_type_from_start_of_msg(data)
         if msg_type is None:
             return len(data)
+        if msg_type == MSG_TIMERS:
+            return self.timer_response_len
         return MSG_LENGTHS[msg_type]
 
     @abstractmethod
@@ -518,7 +525,7 @@ class ProtocolBase:
         """Check if the response is a valid time response."""
         return (
             _message_type_from_start_of_msg(msg) == MSG_TIME
-            and len(msg) == LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN
+            and len(msg) == LEDENET_TIME_RESPONSE_LEN
             and self.is_checksum_correct(msg)
         )
 
@@ -552,6 +559,72 @@ class ProtocolBase:
             )
         )
 
+    def construct_get_timers(self) -> bytearray:
+        """The bytes to get timers."""
+        return self.construct_message(bytearray([0x22, 0x2A, 0x2B, 0x0F]))
+
+    @property
+    def timer_response_len(self) -> int:
+        """Return the time response len."""
+        return LEDENET_TIMERS_8BYTE_RESPONSE_LEN
+
+    @property
+    def timer_len(self) -> int:
+        """Return a single timer len."""
+        return 14
+
+    @property
+    def timer_count(self) -> int:
+        """Return the number of timers."""
+        return 6
+
+    def is_valid_timers_response(self, msg: bytes) -> bool:
+        """Check if the response is a valid timers response."""
+        return (
+            _message_type_from_start_of_msg(msg) == MSG_TIMERS
+            and len(msg) == self.timer_response_len
+            and self.is_checksum_correct(msg)
+        )
+
+    def parse_get_timers(self, msg: bytes) -> List[LedTimer]:
+        """Parse get timers."""
+        if not self.is_valid_timers_response(msg):
+            raise ValueError(f"Timers response not valid: {msg!r}")
+        start = 2
+        timer_list = []
+        timer_bytes_len = self.timer_len
+        # pass in the timer_len-byte timer structs
+        for _ in range(self.timer_count):
+            timer_bytes = msg[start:][:timer_bytes_len]
+            timer = LedTimer(timer_bytes)
+            timer_list.append(timer)
+            start += timer_bytes_len
+        return timer_list
+
+    def construct_set_timers(self, timer_list: List[LedTimer]) -> bytearray:
+        """Construct a set timers message."""
+        # remove inactive or expired timers from list
+        for t in timer_list:
+            t.length = self.timer_len
+            if not t.isActive() or t.isExpired():
+                timer_list.remove(t)
+
+        # truncate if more than 6
+        if len(timer_list) > self.timer_count:
+            _LOGGER.warning("too many timers, truncating list")
+            del timer_list[self.timer_count :]
+
+        # pad list to 6 with inactive timers
+        if len(timer_list) != self.timer_count:
+            for i in range(self.timer_count - len(timer_list)):
+                timer_list.append(LedTimer(length=self.timer_len))
+
+        msg = bytearray([0x21])
+        for t in timer_list:
+            msg.extend(t.toBytes())
+        msg.extend(bytearray([0x00, 0xF0]))
+        return self.construct_message(msg)
+
     def construct_power_restore_state_change(
         self, restore_state: PowerRestoreStates
     ) -> bytearray:
@@ -1162,6 +1235,16 @@ class ProtocolLEDENET9Byte(ProtocolLEDENET8Byte):
         """The name of the protocol."""
         return PROTOCOL_LEDENET_9BYTE
 
+    @property
+    def timer_response_len(self) -> int:
+        """Return the time response len."""
+        return LEDENET_TIMERS_9BYTE_RESPONSE_LEN
+
+    @property
+    def timer_len(self) -> int:
+        """Return a single timer len."""
+        return 15
+
     def construct_levels_change(
         self,
         persist: int,
@@ -1253,6 +1336,16 @@ class ProtocolLEDENET9ByteDimmableEffects(ProtocolLEDENET9ByteAutoOn):
 class ProtocolLEDENETAddressableBase(ProtocolLEDENET9Byte):
     """Base class for addressable protocols."""
 
+    @property
+    def timer_response_len(self) -> int:
+        """Return the time response len."""
+        return LEDENET_TIMERS_8BYTE_RESPONSE_LEN
+
+    @property
+    def timer_len(self) -> int:
+        """Return a single timer len."""
+        return 14
+
 
 class ProtocolLEDENETAddressableA1(ProtocolLEDENETAddressableBase):
     def construct_request_strip_setting(self) -> bytearray:
@@ -1552,7 +1645,7 @@ class ProtocolLEDENETAddressableA2(ProtocolLEDENETAddressableBase):
         #     |  |  |  |  |  |  |  |  |  segments (music mode)
         #     |  |  |  |  |  |  |  |  num pixels (music mode)
         #     |  |  |  |  |  |  |  wiring type (0 indexed, RGB or RGBW)
-        #     |  |  |  |  |  |  ic type (01=WS2812B, 02=SM16703, 03=SM16704, 04=WS2811, 05=UCS1903, 06=SK6812, 07=SK6812RGBW, 08=INK1003, 09=UCS2904B)
+        #     |  |  |  |  |  |  ic type (01=UCS1903, 02=SM16703, 03=WS2811, 04=WS2811B, 05=SK6812, 06=INK1003, 07=WS2801, 08=WS2815, 09=APA102, 10=TM1914, 11=UCS2904B)
         #     |  |  |  |  |  segments
         #     |  |  |  |  ?? (always 0x00)
         #     |  |  |  num pixels (16 bit, low byte)
@@ -1576,21 +1669,17 @@ class ProtocolLEDENETAddressableA2(ProtocolLEDENETAddressableBase):
             hex(segments),
             segments,
         )
-        if NEW_ADDRESSABLE_NUM_TO_OPERATING_MODE.get(msg[6]) == COLOR_MODE_RGBW:
-            wirings = ADDRESSABLE_RGBW_NUM_TO_WIRING
-        else:
-            wirings = ADDRESSABLE_RGB_NUM_TO_WIRING
         return LEDENETAddressableDeviceConfiguration(
             pixels_per_segment=pixels_per_segment,
             segments=segments,
             music_pixels_per_segment=msg[8],
             music_segments=msg[9],
-            wirings=list(wirings.values()),
+            wirings=list(ADDRESSABLE_RGB_NUM_TO_WIRING.values()),
             wiring_num=msg[7],
-            wiring=wirings.get(msg[7]),
-            ic_type=NEW_ADDRESSABLE_NUM_TO_PROTOCOL.get(msg[6]),
+            wiring=ADDRESSABLE_RGB_NUM_TO_WIRING.get(msg[7]),
+            ic_type=A2_NUM_TO_PROTOCOL.get(msg[6]),
             ic_type_num=msg[6],
-            operating_mode=NEW_ADDRESSABLE_NUM_TO_OPERATING_MODE.get(msg[6]),
+            operating_mode=A2_NUM_TO_OPERATING_MODE.get(msg[6]),
         )
 
     def construct_device_config(
@@ -1709,6 +1798,57 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
             inner_pre_constructed=True,
         )
 
+    def parse_strip_setting(self, msg: bytes) -> LEDENETAddressableDeviceConfiguration:
+        """Parse a strip settings message."""
+        # pos  0  1  2  3  4  5  6  7  8  9 10
+        #    00 63 01 2c 00 01 07 08 96 01 45
+        #     |  |  |  |  |  |  |  |  |  |  |
+        #     |  |  |  |  |  |  |  |  |  |  checksum
+        #     |  |  |  |  |  |  |  |  |  |
+        #     |  |  |  |  |  |  |  |  |  segments (music mode)
+        #     |  |  |  |  |  |  |  |  num pixels (music mode)
+        #     |  |  |  |  |  |  |  wiring type (0 indexed, RGB or RGBW)
+        #     |  |  |  |  |  |  ic type (01=WS2812B, 02=SM16703, 03=SM16704, 04=WS2811, 05=UCS1903, 06=SK6812, 07=SK6812RGBW, 08=INK1003, 09=UCS2904B)
+        #     |  |  |  |  |  segments
+        #     |  |  |  |  ?? (always 0x00)
+        #     |  |  |  num pixels (16 bit, low byte)
+        #     |  |  num pixels (16 bit, high byte)
+        #     |  msg head
+        #     msg head
+        #
+        high_byte = msg[2]
+        low_byte = msg[3]
+        pixels_per_segment = (high_byte << 8) + low_byte
+        _LOGGER.debug("bytes: %s", msg)
+        _LOGGER.debug(
+            "Pixel count (high: %s, low: %s) is: %s",
+            hex(high_byte),
+            hex(low_byte),
+            pixels_per_segment,
+        )
+        segments = msg[5]
+        _LOGGER.debug(
+            "Segment count (%s) is: %s",
+            hex(segments),
+            segments,
+        )
+        if NEW_ADDRESSABLE_NUM_TO_OPERATING_MODE.get(msg[6]) == COLOR_MODE_RGBW:
+            wirings = ADDRESSABLE_RGBW_NUM_TO_WIRING
+        else:
+            wirings = ADDRESSABLE_RGB_NUM_TO_WIRING
+        return LEDENETAddressableDeviceConfiguration(
+            pixels_per_segment=pixels_per_segment,
+            segments=segments,
+            music_pixels_per_segment=msg[8],
+            music_segments=msg[9],
+            wirings=list(wirings.values()),
+            wiring_num=msg[7],
+            wiring=wirings.get(msg[7]),
+            ic_type=NEW_ADDRESSABLE_NUM_TO_PROTOCOL.get(msg[6]),
+            ic_type_num=msg[6],
+            operating_mode=NEW_ADDRESSABLE_NUM_TO_OPERATING_MODE.get(msg[6]),
+        )
+
     # To query music mode
     # Send     -> b0 b1 b2 b3 00 01 01 1c 00 03 72 00 72 cb
     # Responds <- b0 b1 b2 b3 00 01 01 1c 00 0d 72 01 26 01 00 00 00 00 00 00 64 64 62 b5
@@ -1898,10 +2038,42 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         )
 
 
+class ProtocolLEDENETSocket(ProtocolLEDENET8Byte):
+    @property
+    def name(self) -> str:
+        """The name of the protocol."""
+        return PROTOCOL_LEDENET_SOCKET
+
+    @property
+    def timer_response_len(self) -> int:
+        """Return the time response len."""
+        return 100
+
+    @property
+    def timer_len(self) -> int:
+        """Return a single timer len."""
+        return 12
+
+    @property
+    def timer_count(self) -> int:
+        """Return the number of timers."""
+        return 8
+
+
 class ProtocolLEDENETCCT(ProtocolLEDENET9Byte):
 
     MIN_BRIGHTNESS = 2
 
+    @property
+    def timer_response_len(self) -> int:
+        """Return the time response len."""
+        return LEDENET_TIMERS_8BYTE_RESPONSE_LEN
+
+    @property
+    def timer_len(self) -> int:
+        """Return a single timer len."""
+        return 14
+
     @property
     def name(self) -> str:
         """The name of the protocol."""
